'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var useLocalStorage = _interopDefault(require('react-use/esm/useLocalStorage'));

/* eslint-env browser */
const getFromPath = (data, path) => path.split(".").reduce((curr, sub) => curr && curr[sub], data);
const defaultObject = Object.freeze({});
const Context = React.createContext({
  lang: "en",
  getMessages: () => defaultObject
});
const messageProxy = {
  set() {
    throw new Error("[use-locales] messages is unvariant.");
  },
  get(obj, prop) {
    return obj[prop] || prop;
  }
};
function LocalesProvider({
  children,
  loadingElm,
  hashKey,
  baseUrl = "/"
}) {
  const [lang, setLang] = useLocalStorage("lang", "en");
  const [locales, setLocales] = useLocalStorage("locales", undefined);
  React.useEffect(() => {
    // load lang
    let currentLang = lang;
    if (!lang) {
      // retrieve lang from navigator
      const {
        navigator
      } = window;
      if (navigator) {
        const {
          language,
          userLanguage,
          languages
        } = navigator;
        currentLang = language || userLanguage || languages && languages.length && languages[0] || "en";
        if (currentLang.length > 2) currentLang = currentLang.substring(0, 2);
        if (currentLang.length < 2) currentLang = "en";
      }
      setLang(currentLang);
    }
    // load locales
    if (!locales || !locales.date || !locales.lang || locales.lang !== currentLang || hashKey && hashKey !== locales.hashKey || Date.now() - locales.date > 259200000 /* 3 days */) {
      const loadLocales = (fetchLang = "en") => fetch(`${baseUrl}locales/${fetchLang}.json${hashKey ? `?hashKey=${hashKey}` : ""}`).then(raw => raw.json()).then(newLocales => setLocales({
        data: newLocales,
        lang: fetchLang,
        date: Date.now(),
        hashKey
      }));
      loadLocales(currentLang).catch(() => {
        setLang("en");
        loadLocales("en");
      });
    }
  }, [hashKey]);
  const getMessages = React.useCallback((path = "") => {
    if (!locales) return defaultObject;
    let {
      data: messages
    } = locales;
    if (path !== "") messages = getFromPath(locales.data, path);
    if (!messages) return defaultObject;
    if (typeof Proxy !== "undefined") return new Proxy(messages, messageProxy);
    return messages;
  }, [locales]);
  const value = React.useMemo(() => ({
    lang: lang !== null && lang !== void 0 ? lang : "en",
    locales,
    getMessages
  }), [lang, locales, getMessages]);
  if (!locales) return loadingElm;
  return React__default.createElement(Context.Provider, {
    value: value
  }, children);
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }
  return keys;
}
function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }
  return target;
}
function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}
function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}
function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return typeof key === "symbol" ? key : String(key);
}

// specs: http://userguide.icu-project.org/formatparse/messages
// inspiration: https://github.com/formatjs/formatjs/tree/master/packages/intl-messageformat
// why redoing it? => bundle size & curiosity!
// this regexp is used to get all pieces {}
const REGEXP_MESSAGE = /{(.+?)}+/g;
// this regexp is used to remove brackets from a piece
const REGEXP_PIECE = /[{}]/g;
function cleanPiece(piece) {
  return piece && piece.replace(REGEXP_PIECE, "").trim();
}
function format(message, data, lang = "en") {
  if (!message || message.trim().length === 0) return message;
  const pieces = message.match(REGEXP_MESSAGE);
  if (!pieces) return message;
  return pieces.reduce((newMessage, piece) => {
    // means we have a key with options (eg: {birthday, date})
    // in this case we don't have options, so this is simple, we replace key by the value
    if (!piece.includes(",")) {
      var _cleanPiece, _data$key;
      const key = (_cleanPiece = cleanPiece(piece)) !== null && _cleanPiece !== void 0 ? _cleanPiece : "";
      return newMessage.replace(piece, (_data$key = data[key]) !== null && _data$key !== void 0 ? _data$key : "");
    }
    // here we have options, we parse the type of options
    // knowing they key is always the first argument, eg in "{birthday, date}" the value key is 'birthday'
    // the second argument is type of options, in previous example this is 'date'
    const pieceOptions = piece.split(",");
    const key = cleanPiece(pieceOptions[0]);
    if (!key || data[key] === null || data[key] === undefined) return newMessage.replace(piece, "");
    const type = cleanPiece(pieceOptions[1]);
    switch (type) {
      case "date":
        {
          // TODO: cache formater
          const value = new Intl.DateTimeFormat(lang).format(new Date(data[key]));
          return newMessage.replace(piece, value);
        }
      case "currency":
        {
          const currency = cleanPiece(pieceOptions[2]);
          // TODO: cache formater
          const value = new Intl.NumberFormat(lang, {
            style: "currency",
            currency
          }).format(data[key]);
          return newMessage.replace(piece, value);
        }
      case "number":
        {
          const value = new Intl.NumberFormat(lang).format(data[key]);
          return newMessage.replace(piece, value);
        }
      default:
        {
          throw new FormatErrorUnknownType(type);
        }
    }
  }, message);
}
class FormatErrorUnknownType extends Error {
  constructor(type) {
    super("messageFormat - Unknown type");
    this.type = type;
  }
}

function useLocales(path, object) {
  const firstDraw = React.useRef(true);
  const {
    getMessages,
    lang
  } = React.useContext(Context);
  const [messages, setMessages] = React.useState(getMessages(path));
  React.useLayoutEffect(() => {
    const rawMessage = getMessages(path);
    if (firstDraw.current && !object) return;
    firstDraw.current = false;
    if (!object) {
      setMessages(rawMessage);
      return;
    }
    setMessages(Object.entries(rawMessage).reduce((acc, [key, value]) => {
      if (typeof value !== "string") return acc;
      return _objectSpread2(_objectSpread2({}, acc), {}, {
        [key]: format(value, object, lang)
      });
    }, {}));
  }, [path, object]);
  return messages;
}

const useLang = () => {
  const {
    lang
  } = React.useContext(Context);
  return lang;
};

exports.LocalesProvider = LocalesProvider;
exports.useLang = useLang;
exports.useLocales = useLocales;
//# sourceMappingURL=index.js.map
